//
// nvWidgets.h - User Interface library
//
//
// Author: Ignacio Castano, Samuel Gateau, Evan Hart
// Email: sdkfeedback@nvidia.com
//
// Copyright (c) NVIDIA Corporation. All rights reserved.
////////////////////////////////////////////////////////////////////////////////
#ifndef NV_WIDGETS_H
#define NV_WIDGETS_H

#include <GL/glut.h>
#include <time.h>   // clock
#include <nvMath.h>

#ifdef WIN32
#ifdef NVWIDGETS_EXPORTS
#define NVSDKENTRY __declspec(dllexport)
#else
#define NVSDKENTRY __declspec(dllimport)
#endif
#endif


namespace nv
{
    struct Point
    {
        NVSDKENTRY Point() : x(0), y(0) {}
        NVSDKENTRY Point(int ix, int iy) : x(ix), y(iy) {}
        NVSDKENTRY Point(const Point & p) : x(p.x), y(p.y) {}

        NVSDKENTRY const Point& operator= (const Point & p) { this->x = p.x; this->y = p.y; return *this; }

        int x, y;
    };

    struct Rect
    {
        NVSDKENTRY Rect() : x(0), y(0), w(0), h(0) {}
        NVSDKENTRY Rect(const Point & p) : x(p.x), y(p.y), w(0), h(0) {}
        NVSDKENTRY Rect(int ix, int iy, int iw = 0, int ih = 0) : x(ix), y(iy), w(iw), h(ih) {}
        NVSDKENTRY Rect(const Rect & r) : x(r.x), y(r.y), w(r.w), h(r.h) {}

        NVSDKENTRY const Rect& operator= (const Rect & r) { this->x = r.x; this->y = r.y; this->w = r.w; this->h = r.h; return *this; }

        int x, y;
        int w, h;
    };

    enum KeyStates
    {
        KEY_OFF = 0x0,
        KEY_ON = 0x1,
        KEY_BEGIN = 0x2,
        KEY_END = 0x4,
        KEY_SHIFT = 0x8,
        KEY_ALT = 0x10,
        KEY_CTRL = 0x20,
    };

    struct KeyState
    {
        int state;
        time_t time;
        Point cursor;
    };

    enum Layout
    {
        LAYOUT_NONE,
        LAYOUT_VERTICAL,
        LAYOUT_HORIZONTAL,
    };

    enum Alignment
    {
        ALIGN_LEFT = 0x01,
        ALIGN_RIGHT = 0x02,
        ALIGN_TOP = 0x04,
        ALIGN_BOTTOM = 0x08,
    };

    enum MouseButton
    {
        MOUSE_LEFT_BUTTON,
        MOUSE_MIDDLE_BUTTON,
        MOUSE_RIGHT_BUTTON,
    };

	// An enum to identify the special key buttons not translated with ASCII codes
	enum KeyboardButton
    {
        KEYBOARD_F1 = 128,
        KEYBOARD_F2,
        KEYBOARD_F3,
        KEYBOARD_F4,
        KEYBOARD_F5,
        KEYBOARD_F6,
        KEYBOARD_F7,
        KEYBOARD_F8,
        KEYBOARD_F9,
        KEYBOARD_F10,
        KEYBOARD_F11,
        KEYBOARD_F12,

        KEYBOARD_LEFT,
        KEYBOARD_UP,
        KEYBOARD_RIGHT,
        KEYBOARD_DOWN,
        KEYBOARD_PAGE_UP,
        KEYBOARD_PAGE_DOWN,
        KEYBOARD_HOME,
        KEYBOARD_END,
        KEYBOARD_INSERT,
    };

    enum MouseState
    {
        MOUSE_DOWN,
        MOUSE_UP
    };

    struct Canvas
    {
        Rect bounds;  // anchor point + width and height of the region
        Layout layout;
        int alignment;
        int margin; // border 
        int space;  // interior
    };


    //*************************************************************************
    // The various flags to modify the style in which are executed the widgets   
    enum StyleFlag
        {
        };


    //*************************************************************************
    // UIPainter
    class UIPainter
    {
    public:
        NVSDKENTRY UIPainter() {}

        NVSDKENTRY virtual void begin( const Rect& window ) { init(); }
        NVSDKENTRY virtual void end() {}

		// These methods should be called between begin/end
        NVSDKENTRY virtual Rect getLabelRect(const Rect & r, const char * text, Rect & rt, int& nbLines) const = 0;
        NVSDKENTRY virtual void drawLabel(const Rect & r, const char * text, const Rect & rt, const int& nbLines, bool isHover, int style) = 0;
      
        NVSDKENTRY virtual Rect getButtonRect(const Rect & r, const char * text, Rect & rt) const = 0;
		NVSDKENTRY virtual void drawButton(const Rect & r, const char * text, const Rect & rt, bool isDown, bool isHover, bool isFocus, int style) = 0;
    
		NVSDKENTRY virtual Rect getCheckRect(const Rect & r, const char * text, Rect & rt, Rect & rc) const = 0;
        NVSDKENTRY virtual void drawCheckButton(const Rect & r, const char * text, const Rect & rt, const Rect & rr, bool isChecked, bool isHover, bool isFocus, int style) = 0;

		NVSDKENTRY virtual Rect getRadioRect(const Rect & r, const char * text, Rect & rt, Rect & rr) const = 0;
        NVSDKENTRY virtual void drawRadioButton(const Rect & r, const char * text, const Rect & rt, const Rect & rr, bool isOn, bool isHover, bool isFocus, int style) = 0;

		NVSDKENTRY virtual Rect getHorizontalSliderRect(const Rect & r, Rect& rs, float v, Rect& rc) const = 0;
        NVSDKENTRY virtual void drawHorizontalSlider(const Rect & r, Rect& rs, float v, Rect& rc, bool isHover, int style) = 0;

		NVSDKENTRY virtual Rect getItemRect(const Rect & r, const char * text, Rect & rt) const = 0;
        NVSDKENTRY virtual void drawListItem(const Rect & r, const char * text, const Rect & rt, bool isSelected, bool isHover, int style) = 0;
 
        NVSDKENTRY virtual Rect getListRect(const Rect & r, int numOptions, const char * options[], Rect& ri, Rect & rt) const = 0;
		NVSDKENTRY virtual void drawListBox(const Rect & r, int numOptions, const char * options[], const Rect& ri, const Rect & rt, int selected, int hovered, int style) = 0;
      
		NVSDKENTRY virtual Rect getComboRect(const Rect & r, int numOptions, const char * options[], int selected, Rect& rt, Rect& ra) const = 0;
        NVSDKENTRY virtual Rect getComboOptionsRect(const Rect & rCombo, int numOptions, const char * options[], Rect& ri, Rect & rit) const = 0;
		NVSDKENTRY virtual void drawComboBox(const Rect & rect, int numOptions, const char * options[], const Rect & rt, int selected, bool isHover, bool isFocus, int style) = 0;
        NVSDKENTRY virtual void drawComboOptions(const Rect & rect, int numOptions, const char * options[], const Rect& ri, const Rect & rit, int selected, int hovered, bool isHover, bool isFocus, int style) = 0;

		NVSDKENTRY virtual Rect getLineEditRect(const Rect & r, const char * text, Rect & rt) const = 0;
        NVSDKENTRY virtual void drawLineEdit(const Rect & r, const char * text, const Rect & rt, int caretPos, bool isSelected, bool isHover, int style) = 0;

		// Eval widget dimensions
		NVSDKENTRY virtual int getCanvasMargin() const = 0;
        NVSDKENTRY virtual int getCanvasSpace() const = 0;		
		NVSDKENTRY virtual int getFontHeight() const = 0;
        NVSDKENTRY virtual int getTextLineWidth(const char * text) const = 0;		
        NVSDKENTRY virtual int getTextSize(const char * text, int& nbLines) const = 0;		
		NVSDKENTRY virtual int getPickedCharNb(const char * text, const Point& at) const = 0;		

		NVSDKENTRY virtual void drawDebugRect(const Rect & r) = 0;

    protected:
    
        NVSDKENTRY void init() {}
    };


    class UIContext
    {
    public:
        NVSDKENTRY UIContext( UIPainter& painter );

        NVSDKENTRY void reshape(int w, int h);
        NVSDKENTRY void mouse(int button, int state, int modifier, int x, int y);
        NVSDKENTRY void mouse(int button, int state, int x, int y) { mouse( button, state, 0, x, y); }
        NVSDKENTRY void motion(int x, int y);
        NVSDKENTRY void keyboardDown(unsigned char k, int x, int y);
        NVSDKENTRY void keyboardUp(unsigned char k, int x, int y);

        NVSDKENTRY bool isOnFocus() const { return m_uiOnFocus; }

        NVSDKENTRY void begin();
        NVSDKENTRY void end();

        // These methods should be called between begin/end
        NVSDKENTRY void doLabel(const Rect & rect, const char * text, int style = 0);
        NVSDKENTRY bool doButton(const Rect & rect, const char * text, bool * state = NULL, int style = 0);
        
		NVSDKENTRY bool doCheckButton(const Rect & rect, const char * text, bool * state, int style = 0);

		NVSDKENTRY bool doRadioButton(int reference, const Rect & r, const char * text, int * value, int style = 0);

        NVSDKENTRY bool doHorizontalSlider(const Rect & rect, float min, float max, float * value, int style = 0);

        NVSDKENTRY bool doListItem(int index, const Rect & rect, const char * text, int * selected, int style = 0);
        NVSDKENTRY bool doListBox(const Rect & rect, int numOptions, const char * options[], int * selected, int style = 0);
        NVSDKENTRY bool doComboBox(const Rect & rect, int numOptions, const char * options[], int * selected, int style = 0);

        NVSDKENTRY bool doLineEdit(const Rect & r, char * text, int maxTextLength, int * nbCharsReturned, int style = 0);

		NVSDKENTRY void setLayout(Layout layout, int alignment);
        
        NVSDKENTRY void beginGroup();
        NVSDKENTRY void endGroup();

        NVSDKENTRY int getCursorX() { return m_currentCursor.x;}
        NVSDKENTRY int getCursorY() { return m_currentCursor.y;}

        NVSDKENTRY const KeyState& getMouseState( int button) { return m_mouseButton[button]; }

    private:
        NVSDKENTRY void setCursor(int x, int y);

        NVSDKENTRY static bool overlap(const Rect & rect, const Point & p);

        NVSDKENTRY bool hasFocus(const Rect & rect);
        NVSDKENTRY bool isHover(const Rect & rect);

        NVSDKENTRY Rect placeRect(const Rect & r);

    private:
        UIPainter * m_painter;
        
        int m_canvasIndex;
        Canvas m_canvasStack[64];

        Rect m_window;

        Point m_currentCursor;
        KeyState m_keyboard[256];
        KeyState m_mouseButton[3];
		unsigned char m_keyBuffer[32];
		int m_nbKeys;

		int m_focusCaretPos;
		Point m_focusPoint;
		bool m_2StepFocus;
        bool m_uiOnFocus;
    };

};





#endif  // NV_WIDGETS_H
